@@ -2,10 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2022 */
 /* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* This code originally came from Robert Wal. */
 
@@ -65,16 +64,13 @@ switch (wso_query(CS query, CS buffer, sizeof(buffer)))
 
 #include "../version.h"
 
-gstring *
-whoson_version_report(gstring * g)
+void
+whoson_version_report(FILE *f)
 {
-g = string_fmt_append(g,
-  "Library version: Whoson: Runtime: %s\n", wso_version());
+fprintf(f, "Library version: Whoson: Runtime: %s\n", wso_version());
 #ifdef DYNLOOKUP
-g = string_fmt_append(g,
-  "                         Exim version %s\n", EXIM_VERSION_STR);
+fprintf(f, "                         Exim version %s\n", EXIM_VERSION_STR);
 #endif
-return g;
 }
 
 static lookup_info _lookup_info = {
