@@ -2,15 +2,12 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 #include "../exim.h"
-
-#ifdef ROUTER_IPLOOKUP		/* Remainder of file */
 #include "rf_functions.h"
 #include "iplookup.h"
 
@@ -53,7 +50,7 @@ int iplookup_router_options_count =
 
 /* Dummy entries */
 iplookup_router_options_block iplookup_router_option_defaults = {0};
-void iplookup_router_init(driver_instance *rblock) {}
+void iplookup_router_init(router_instance *rblock) {}
 int iplookup_router_entry(router_instance *rblock, address_item *addr,
   struct passwd *pw, int verify, address_item **addr_local,
   address_item **addr_remote, address_item **addr_new,
@@ -87,37 +84,36 @@ iplookup_router_options_block iplookup_router_option_defaults = {
 consistency checks to be done, or anything else that needs to be set up. */
 
 void
-iplookup_router_init(driver_instance * r)
+iplookup_router_init(router_instance *rblock)
 {
-router_instance * rblock = (router_instance *)r;
-iplookup_router_options_block * ob =
-  (iplookup_router_options_block *) r->options_block;
+iplookup_router_options_block *ob =
+  (iplookup_router_options_block *)(rblock->options_block);
 
 /* A port and a host list must be given */
 
 if (ob->port < 0)
-  log_write_die(0, LOG_CONFIG_FOR, "%s router:\n  "
-    "a port must be specified", r->name);
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
+    "a port must be specified", rblock->name);
 
-if (!ob->hosts)
-  log_write_die(0, LOG_CONFIG_FOR, "%s router:\n  "
-    "a host list must be specified", r->name);
+if (ob->hosts == NULL)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
+    "a host list must be specified", rblock->name);
 
 /* Translate protocol name into value */
 
-if (ob->protocol_name)
+if (ob->protocol_name != NULL)
   {
   if (Ustrcmp(ob->protocol_name, "udp") == 0) ob->protocol = ip_udp;
   else if (Ustrcmp(ob->protocol_name, "tcp") == 0) ob->protocol = ip_tcp;
-  else log_write_die(0, LOG_CONFIG_FOR, "%s router:\n  "
-    "protocol not specified as udp or tcp", r->name);
+  else log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
+    "protocol not specified as udp or tcp", rblock->name);
   }
 
 /* If a response pattern is given, compile it now to get the error early. */
 
-if (ob->response_pattern)
+if (ob->response_pattern != NULL)
   ob->re_response_pattern =
-    regex_must_compile(ob->response_pattern, MCS_NOFLAGS, TRUE);
+    regex_must_compile(ob->response_pattern, FALSE, TRUE);
 }
 
 
@@ -165,36 +161,38 @@ uschar *reply;
 uschar *hostname, *reroute, *domain;
 const uschar *listptr;
 uschar host_buffer[256];
-host_item *host = store_get(sizeof(host_item), GET_UNTAINTED);
+host_item *host = store_get(sizeof(host_item), FALSE);
 address_item *new_addr;
 iplookup_router_options_block *ob =
-  (iplookup_router_options_block *)(rblock->drinst.options_block);
-const pcre2_code *re = ob->re_response_pattern;
+  (iplookup_router_options_block *)(rblock->options_block);
+const pcre *re = ob->re_response_pattern;
 int count, query_len, rc;
 int sep = 0;
 
-DEBUG(D_route) debug_printf_indent("%s router called for %s: domain = %s\n",
-  rblock->drinst.name, addr->address, addr->domain);
+DEBUG(D_route) debug_printf("%s router called for %s: domain = %s\n",
+  rblock->name, addr->address, addr->domain);
 
-reply = store_get(256, GET_TAINTED);
+reply = store_get(256, TRUE);	/* tainted data */
 
 /* Build the query string to send. If not explicitly given, a default of
 "user@domain user@domain" is used. */
 
-GET_OPTION("query");
-if (!ob->query)
+if (ob->query == NULL)
   query = string_sprintf("%s@%s %s@%s", addr->local_part, addr->domain,
     addr->local_part, addr->domain);
 else
-  if (!(query = expand_string(ob->query)))
+  {
+  query = expand_string(ob->query);
+  if (query == NULL)
     {
     addr->message = string_sprintf("%s router: failed to expand %s: %s",
-      rblock->drinst.name, ob->query, expand_string_message);
+      rblock->name, ob->query, expand_string_message);
     return DEFER;
     }
+  }
 
 query_len = Ustrlen(query);
-DEBUG(D_route) debug_printf("%s router query is %q\n", rblock->drinst.name,
+DEBUG(D_route) debug_printf("%s router query is \"%s\"\n", rblock->name,
   string_printing(query));
 
 /* Now connect to the required port for each of the hosts in turn, until a
@@ -239,7 +237,7 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
     /* Create a socket, for UDP or TCP, as configured. IPv6 addresses are
     detected by checking for a colon in the address. */
 
-    host_af = Ustrchr(h->address, ':') ? AF_INET6 : AF_INET;
+    host_af = (Ustrchr(h->address, ':') != NULL)? AF_INET6 : AF_INET;
 
     query_cctx.sock = ip_socket(ob->protocol == ip_udp ? SOCK_DGRAM:SOCK_STREAM,
       host_af);
@@ -247,7 +245,7 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
       {
       if (ob->optional) return PASS;
       addr->message = string_sprintf("failed to create socket in %s router",
-        rblock->drinst.name);
+        rblock->name);
       return DEFER;
       }
 
@@ -291,8 +289,8 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
     /* Success; break the loop */
 
     reply[count] = 0;
-    DEBUG(D_route) debug_printf("%s router received %q from %s\n",
-      rblock->drinst.name, string_printing(reply), h->address);
+    DEBUG(D_route) debug_printf("%s router received \"%s\" from %s\n",
+      rblock->name, string_printing(reply), h->address);
     break;
     }
 
@@ -308,12 +306,12 @@ connect to any of the IP addresses, or timed out while reading or writing to
 those we have connected to. In all cases, we must pass if optional and
 defer otherwise. */
 
-if (!hostname)
+if (hostname == NULL)
   {
-  DEBUG(D_route) debug_printf("%s router failed to get anything\n", rblock->drinst.name);
+  DEBUG(D_route) debug_printf("%s router failed to get anything\n", rblock->name);
   if (ob->optional) return PASS;
   addr->message = string_sprintf("%s router: failed to communicate with any "
-    "host", rblock->drinst.name);
+    "host", rblock->name);
   return DEFER;
   }
 
@@ -327,7 +325,7 @@ if (re != NULL)
   if (!regex_match_and_setup(re, reply, 0, -1))
     {
     DEBUG(D_route) debug_printf("%s router: %s failed to match response %s\n",
-      rblock->drinst.name, ob->response_pattern, reply);
+      rblock->name, ob->response_pattern, reply);
     return DECLINE;
     }
   }
@@ -352,7 +350,7 @@ else
     if (Ustrcmp(query + query_len/2 + 1, reply+nn) != 0)
       {
       DEBUG(D_route) debug_printf("%s router: failed to match identification "
-        "in response %s\n", rblock->drinst.name, reply);
+        "in response %s\n", rblock->name, reply);
       return DECLINE;
       }
     }
@@ -363,29 +361,28 @@ else
 /* If an explicit rerouting string is specified, expand it. Otherwise, use
 what was sent back verbatim. */
 
-GET_OPTION("reroute");
-if (ob->reroute)
+if (ob->reroute != NULL)
   {
   reroute = expand_string(ob->reroute);
   expand_nmax = -1;
-  if (!reroute)
+  if (reroute == NULL)
     {
     addr->message = string_sprintf("%s router: failed to expand %s: %s",
-      rblock->drinst.name, ob->reroute, expand_string_message);
+      rblock->name, ob->reroute, expand_string_message);
     return DEFER;
     }
   }
-else
-  reroute = reply;
+else reroute = reply;
 
 /* We should now have a new address in the form user@domain. */
 
-if (!(domain = Ustrchr(reroute, '@')))
+domain = Ustrchr(reroute, '@');
+if (domain == NULL)
   {
   log_write(0, LOG_MAIN, "%s router: reroute string %s is not of the form "
-    "user@domain", rblock->drinst.name, reroute);
+    "user@domain", rblock->name, reroute);
   addr->message = string_sprintf("%s router: reroute string %s is not of the "
-    "form user@domain", rblock->drinst.name, reroute);
+    "form user@domain", rblock->name, reroute);
   return DEFER;
   }
 
@@ -398,8 +395,8 @@ new_addr->parent = addr;
 new_addr->prop = addr->prop;
 
 if (addr->child_count == USHRT_MAX)
-  log_write_die(0, LOG_MAIN, "%s router generated more than %d "
-    "child addresses for <%s>", rblock->drinst.name, USHRT_MAX, addr->address);
+  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s router generated more than %d "
+    "child addresses for <%s>", rblock->name, USHRT_MAX, addr->address);
 addr->child_count++;
 new_addr->next = *addr_new;
 *addr_new = new_addr;
@@ -417,30 +414,5 @@ if (rc != OK) return rc;
 return OK;
 }
 
-
-
-# ifdef DYNLOOKUP
-#  define iplookup_router_info _router_info
-# endif
-#endif	/*!MACRO_PREDEF*/
-
-router_info iplookup_router_info =
-{
-.drinfo = {
-  .driver_name =	US"iplookup",
-  .options =		iplookup_router_options,
-  .options_count =	&iplookup_router_options_count,
-  .options_block =	&iplookup_router_option_defaults,
-  .options_len =	sizeof(iplookup_router_options_block),
-  .init =		iplookup_router_init,
-# ifdef DYNLOOKUP
-  .dyn_magic =		ROUTER_MAGIC,
-# endif
-  },
-.code =			iplookup_router_entry,
-.tidyup =		NULL,     /* no tidyup entry */
-.ri_flags =		ri_notransport
-};
-
-#endif	/*ROUTER_IPLOOKUP*/
+#endif   /*!MACRO_PREDEF*/
 /* End of routers/iplookup.c */
