@@ -2,10 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) The Exim Maintainers 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 #include "../exim.h"
@@ -50,9 +49,9 @@ Returns:               OK
 */
 
 int
-rf_lookup_hostlist(router_instance * rblock, address_item * addr,
-  const uschar * ignore_target_hosts, int lookup_type, int hff_code,
-  address_item ** addr_new)
+rf_lookup_hostlist(router_instance *rblock, address_item *addr,
+  uschar *ignore_target_hosts, int lookup_type, int hff_code,
+  address_item **addr_new)
 {
 BOOL self_send = FALSE;
 
@@ -72,7 +71,7 @@ for (host_item * prev = NULL, * h = addr->host_list, *next_h; h; h = next_h)
   if (h->address) { prev = h; continue; }
 
   DEBUG(D_route|D_host_lookup)
-    debug_printf_indent("finding IP address for %s\n", h->name);
+    debug_printf("finding IP address for %s\n", h->name);
 
   /* Handle any port setting that may be on the name; it will be removed
   from the end of the name. */
@@ -119,7 +118,7 @@ for (host_item * prev = NULL, * h = addr->host_list, *next_h; h; h = next_h)
 
   else if (lookup_type & LK_BYNAME || string_is_ip_address(h->name, NULL) != 0)
     {
-    DEBUG(D_route|D_host_lookup) debug_printf_indent("calling host_find_byname\n");
+    DEBUG(D_route|D_host_lookup) debug_printf("calling host_find_byname\n");
     rc = host_find_byname(h, ignore_target_hosts, HOST_FIND_QUALIFY_SINGLE,
       &canonical_name, TRUE);
     }
@@ -174,7 +173,7 @@ for (host_item * prev = NULL, * h = addr->host_list, *next_h; h; h = next_h)
       {
       DEBUG(D_route)
         debug_printf("%s router timed out and pass_on_timeout set\n",
-          rblock->drinst.name);
+          rblock->name);
       return PASS;
       }
     addr->message = string_sprintf("host lookup for %s did not complete "
@@ -198,8 +197,8 @@ for (host_item * prev = NULL, * h = addr->host_list, *next_h; h; h = next_h)
 
     addr->basic_errno = ERRNO_UNKNOWNHOST;
     addr->message =
-      string_sprintf("lookup of host %q failed in %s router%s",
-        h->name, rblock->drinst.name,
+      string_sprintf("lookup of host \"%s\" failed in %s router%s",
+        h->name, rblock->name,
         f.host_find_failed_syntax? ": syntax error in name" : "");
 
     if (hff_code == hff_defer) return DEFER;
